import hashlib
import zlib


def get_filename_without_ending(file_path: str) -> str:
    """
    Returns the filename without extension
    :param file_path:
    :return:
    """

    # if filename has file_path parts
    if '/' in file_path:
        filename = file_path.rsplit('/')[-1]
    else:
        filename = file_path

    return filename.rsplit('.', 1)[0]


def crc(file_path: str) -> str:
    """Calculates the cyclic redundancy checksum (CRC) of the given file.

    See ``open`` for all the exceptins that can be raised.

    :param file_path: the file for which the CRC checksum should be calculated.
    :return: returns the CRC checksum of the file in hexadecimal format (8 characters).
    """
    prev = 0
    with open(file_path, "rb") as f:
        for line in f:
            prev = zlib.crc32(line, prev)
    return "%08X" % (prev & 0xFFFFFFFF)


def md5_checksum(file_path: str) -> str:
    """
    Returns the md5 checksum of the file from the given file_path.

    See ``open`` for all the exceptins that can be raised.

    :param file_path: the file for which the MD5 hashsum should be calculated.
    :return: returns the MD5 of the file in hexadecimal format.
    """
    with open(file_path, 'rb') as fh:
        m = hashlib.md5()
        while True:
            data = fh.read(8192)
            if not data:
                break
            m.update(data)
        return m.hexdigest()



from django.test import TestCase

from ai_django_core.sentry.helpers import strip_sensitive_data_from_sentry_event


class SentryHelperTest(TestCase):

    def test_strip_sensitive_data_from_sentry_event_regular(self):
        event = {'user': {'email': 'mymail@example.com', 'ip_address': '127.0.0.1', 'username': 'my-user'}}

        self.assertIsInstance(strip_sensitive_data_from_sentry_event(event, None), dict)

    def test_strip_sensitive_data_from_sentry_event_missing_key_email(self):
        event = {'user': {'ip_address': '127.0.0.1', 'username': 'my-user'}}

        self.assertIsInstance(strip_sensitive_data_from_sentry_event(event, None), dict)

    def test_strip_sensitive_data_from_sentry_event_missing_key_ip_address(self):
        event = {'user': {'email': 'mymail@example.com', 'username': 'my-user'}}

        self.assertIsInstance(strip_sensitive_data_from_sentry_event(event, None), dict)

    def test_strip_sensitive_data_from_sentry_event_missing_key_username(self):
        event = {'user': {'email': 'mymail@example.com', 'ip_address': '127.0.0.1'}}

        self.assertIsInstance(strip_sensitive_data_from_sentry_event(event, None), dict)



import bleach


class BleacherMixin:
    """
    Removes HTML tags and attributes from the fields defined in :py:attr:`BLEACH_FIELD_LIST`.
    Allowed tags and attributes are defined in the :py:attr:`ALLOWED_TAGS` and :py:attr:`ALLOWED_ATTRIBUTES` attributes.
    If :py:attr:`ALLOWED_TAGS` and :py:attr:`ALLOWED_ATTRIBUTES` are not specified on a model a set of default value is
    used:

    ALLOWED_TAGS:
      * span
      * p
      * h1
      * h2
      * h3
      * h4
      * h5
      * h6
      * img
      * div
      * u
      * br
      * blockquote

    ALLOWED_ATTRIBUTES:
      * all tags: class, style, id
      * a: href, rel
      * img: alt, src
    """

    BLEACH_FIELD_LIST = []

    DEFAULT_ALLOWED_ATTRIBUTES = {
        '*': ['class', 'style', 'id'],
        'a': ['href', 'rel'],
        'img': ['alt', 'src'],
    }

    DEFAULT_ALLOWED_TAGS = bleach.ALLOWED_TAGS + ['span', 'p', 'h1', 'h2', 'h3', 'h4', 'h5',
                                                  'h6', 'img', 'div', 'u', 'br', 'blockquote']

    def __init__(self, *args, **kwargs):
        super().__init__(*args, **kwargs)
        self.fields_to_bleach = getattr(self, 'BLEACH_FIELD_LIST', [])
        self.allowed_tags = getattr(self, 'ALLOWED_TAGS', self.DEFAULT_ALLOWED_TAGS)
        self.allowed_attributes = getattr(self, 'ALLOWED_ATTRIBUTES', self.DEFAULT_ALLOWED_ATTRIBUTES)

    def _bleach_field(self, field_name):
        str_to_bleach = getattr(self, field_name, '')
        if str_to_bleach:
            cleaned_value = bleach.clean(str_to_bleach, tags=self.allowed_tags, attributes=self.allowed_attributes)
            setattr(self, field_name, cleaned_value)

    def save(self, *args, **kwargs):
        for field in self.fields_to_bleach:
            self._bleach_field(field)

        super().save(*args, **kwargs)



from django import forms
from django.forms import BaseInlineFormSet, inlineformset_factory
from django.test import TestCase

from ai_django_core.view_layer.formset_mixins import CountChildrenFormsetMixin
from testapp.models import MySingleSignalModel, ForeignKeyRelatedModel


class MySingleSignalModelForm(forms.ModelForm):
    class Meta:
        model = MySingleSignalModel
        exclude = ()


class MySingleSignalModelFormset(CountChildrenFormsetMixin, BaseInlineFormSet):
    pass


class CountChildrenFormsetMixinTest(TestCase):

    def test_simple_no_data(self):
        formset_class = inlineformset_factory(MySingleSignalModel, ForeignKeyRelatedModel,
                                              form=MySingleSignalModelForm,
                                              formset=MySingleSignalModelFormset,
                                              extra=3, can_delete=True, max_num=3)

        formset = formset_class()
        self.assertEqual(formset.get_number_of_children(), 0)

    def test_regular_with_data(self):
        mssm = MySingleSignalModel.objects.create(value=27)
        ForeignKeyRelatedModel.objects.create(single_signal=mssm)
        ForeignKeyRelatedModel.objects.create(single_signal=mssm)

        formset_class = inlineformset_factory(MySingleSignalModel, ForeignKeyRelatedModel,
                                              form=MySingleSignalModelForm,
                                              formset=MySingleSignalModelFormset,
                                              extra=3, can_delete=True, max_num=3)

        formset = formset_class(
            {'fkrm-INITIAL_FORMS': '2',
             'fkrm-MIN_NUM_FORMS': '2',
             'fkrm-MAX_NUM_FORMS': '3',
             'fkrm-TOTAL_FORMS': '2'
             },
            None,
            instance=mssm, prefix='fkrm')

        formset.is_valid()
        self.assertEqual(formset.get_number_of_children(), 2)



from django.db.models import QuerySet, Manager


class AbstractPermissionMixin:
    """
    Mixin that provides an interface for a basic per-object permission system.
    Single objects cannot be checked individually, but can be matched with the corresponding query set.
    Please append further methods here, if necessary, in order to make them accessible at all inheriting classes
    (query sets AND managers).
    """

    def visible_for(self, user):
        raise NotImplementedError('Please implement this method')

    def editable_for(self, user):
        raise NotImplementedError('Please implement this method')

    def deletable_for(self, user):
        raise NotImplementedError('Please implement this method')


class AbstractUserSpecificQuerySet(QuerySet, AbstractPermissionMixin):
    """"
    Extend this queryset in your model if you want to implement a visible_for functionality.
    """
    def default(self, user):
        return self

    def visible_for(self, user):
        raise NotImplementedError('Please implement this method')

    def editable_for(self, user):
        raise NotImplementedError('Please implement this method')

    def deletable_for(self, user):
        raise NotImplementedError('Please implement this method')


class AbstractUserSpecificManager(Manager, AbstractPermissionMixin):
    """
    The UserSpecificQuerySet has a method 'as_manger', which can be used for creating a default manager,
    which inherits all methods of the queryset and invokes the respective method of it's queryset, respectively.
    If the manager has to be declared separately for some reasons, all queryset methods, have to be declared twice,
    once in the QuerySet, once in the manager class.
    For consistency reasons, both inherit from the same mixin, to ensure the equality of the method's names.
    """

    def visible_for(self, user):
        return self.get_queryset().visible_for(user)

    def editable_for(self, user):
        return self.get_queryset().editable_for(user)

    def deletable_for(self, user):
        return self.get_queryset().deletable_for(user)


class GloballyVisibleQuerySet(AbstractUserSpecificQuerySet):
    """
    Manager (QuerySet) for classes which do NOT have any visibility restrictions.
    """

    def visible_for(self, user):
        return self.all()

    def editable_for(self, user):
        return self.visible_for(user)

    def deletable_for(self, user):
        return self.visible_for(user)



from django.core.mail import EmailMultiAlternatives
from django.db import models
from django.db.models.signals import post_save, pre_save
from django.dispatch import receiver

from ai_django_core.managers import GloballyVisibleQuerySet
from ai_django_core.models import CommonInfo


class MySingleSignalModel(models.Model):
    value = models.PositiveIntegerField(default=0)

    objects = GloballyVisibleQuerySet.as_manager()


class ForeignKeyRelatedModel(models.Model):
    single_signal = models.ForeignKey(MySingleSignalModel, on_delete=models.CASCADE,
                                      related_name='foreign_key_related_models')

    objects = GloballyVisibleQuerySet.as_manager()


@receiver(pre_save, sender=MySingleSignalModel)
def increase_value_no_dispatch_uid(sender, instance, **kwargs):
    instance.value += 1


class MyMultipleSignalModel(models.Model):
    value = models.PositiveIntegerField(default=0)


@receiver(pre_save, sender=MyMultipleSignalModel, dispatch_uid='test.mysinglesignalmodel.increase_value_with_uuid')
def increase_value_with_dispatch_uid(sender, instance, **kwargs):
    instance.value += 1


@receiver(post_save, sender=MyMultipleSignalModel)
def send_email(sender, instance, **kwargs):
    msg = EmailMultiAlternatives('Test Mail', 'I am content',
                                 from_email='test@example.com',
                                 to=['random.dude@example.com'])
    msg.send()


class CommonInfoBasedModel(CommonInfo):
    value = models.PositiveIntegerField(default=0)


class ModelWithFkToSelf(models.Model):
    parent = models.ForeignKey('self', blank=True, null=True, related_name='children', on_delete=models.CASCADE)


class ModelWithOneToOneToSelf(models.Model):
    peer = models.OneToOneField('self', blank=True, null=True, related_name='related_peer', on_delete=models.CASCADE)



from django.contrib import admin
from django.contrib.auth.models import User
from django.test import TestCase

from ai_django_core.admin.model_admins.mixins import AdminNoInlinesForCreateMixin
from ai_django_core.tests.mixins import RequestProviderMixin
from testapp.models import ForeignKeyRelatedModel, MySingleSignalModel


class ForeignKeyRelatedModelTabularInline(admin.TabularInline):
    model = ForeignKeyRelatedModel


class TestAdminNoInlinesForCreateMixinAdmin(AdminNoInlinesForCreateMixin, admin.ModelAdmin):
    inlines = (ForeignKeyRelatedModelTabularInline,)


class AdminNoInlinesForCreateMixinTest(RequestProviderMixin, TestCase):

    @classmethod
    def setUpTestData(cls):
        super().setUpTestData()

        cls.super_user = User.objects.create(username='super_user', is_superuser=True)

        admin.site.register(MySingleSignalModel, TestAdminNoInlinesForCreateMixinAdmin)

    @classmethod
    def tearDownClass(cls):
        super().tearDownClass()

        admin.site.unregister(MySingleSignalModel)

    def test_inlines_are_removed_on_create(self):
        model_admin = TestAdminNoInlinesForCreateMixinAdmin(model=MySingleSignalModel, admin_site=admin.site)

        self.assertEqual(model_admin.get_inline_instances(
            self.get_request(self.super_user), obj=None), [])

    def test_inlines_are_not_removed_on_edit(self):
        model_admin = TestAdminNoInlinesForCreateMixinAdmin(model=MySingleSignalModel, admin_site=admin.site)

        self.assertEqual(len(model_admin.get_inline_instances(
            self.get_request(self.super_user), obj=MySingleSignalModel(value=1))), 1)
